﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.BioBoost.Utils {

  public static class ValueParameterCollectionExtensions {
    /// <summary>
    /// total conversion cost including costs for required supplies minus revenue for side products.
    /// </summary>
    /// <param name="productsAndAmounts">the list of products and amounts</param>
    /// <param name="productPrices">a dictionary containing the prices to use</param>
    /// <param name="mainProductName">the main product i.e. no side product and hence excluded from this calculation</param>
    /// <returns></returns>
    public static double TotalCost(this ValueParameterCollection productsAndAmounts, IDictionary<string, double> productPrices, string mainProductName) {
      double cost = 0;
      foreach (var p in productsAndAmounts) {
        var productName = p.Name;
        var amount = p.Value as DoubleValue;
        double price;
        if (amount != null && productName != mainProductName && productPrices.TryGetValue(p.Name, out price)) {
          cost -= amount.Value*price; // as amount*price is sales revenue -amount*price is cost ;-)
        }
      }
      return cost;
    } 
  }
}
